local insetValue = 2;

CUI_FRAME_BACKGROUND = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-background";
CUI_FRAME_GRADIENT_BACKGROUND = "Interface\\AddOns\\CleanUI\\skins\\frame\\gradient-background";

CUI_FRAME_BORDER = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-border";
CUI_FRAME_BORDER_WIDE = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-border-wide";

function CleanUI_RemoveBackdrop(frame)
    frame:SetBackdrop(nil);
    local backdropFrame = _G[frame:GetName().."BackdropFrame"];
    if (backdropFrame) then
        backdropFrame:SetBackdrop(nil);
    end
end

function CleanUI_SetGradientBackdrop(frame, top, left, bottom, right)
    local parent = frame;

    if (top or left or bottom or right) then
        local backdropFrame = _G[frame:GetName().."BackdropFrame"];
        if (not backdropFrame) then
            backdropFrame = CreateFrame("Frame", frame:GetName().."BackdropFrame", frame);
            backdropFrame:SetParent(frame);
            backdropFrame:SetFrameStrata("BACKGROUND");
        end

        backdropFrame:ClearAllPoints();
        backdropFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", (left or 0), (top or 0));
        backdropFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", (right or 0), (bottom or 0));

        parent = backdropFrame;
    end

    parent:SetBackdrop( {
        bgFile = CUI_FRAME_GRADIENT_BACKGROUND,
        tile = false,
        insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
    });
end

function CleanUI_SetBackdrop(frame, addBackground, borderType, top, left, bottom, right)
    local backdropFrame = _G[frame:GetName().."BackdropFrame"];
    if (not backdropFrame) then
        backdropFrame = CreateFrame("Frame", frame:GetName().."BackdropFrame", frame, BackdropTemplateMixin and "BackdropTemplate");
        backdropFrame:SetParent(frame);
        backdropFrame:SetFrameStrata("BACKGROUND");
    end

    backdropFrame:ClearAllPoints();
    backdropFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", (left or 0), (top or 0));
    backdropFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", (right or 0), (bottom or 0));

    local parent = backdropFrame;

    local border = CUI_FRAME_BORDER;

    if (borderType and borderType == "wide") then
        border = CUI_FRAME_BORDER_WIDE;
    end


    if (addBackground == false) then
        parent:SetBackdrop( {
            edgeFile = border, tile = true, tileSize = 64, edgeSize = 16,
            insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
        });
    else
        parent:SetBackdrop( {
            bgFile = CUI_FRAME_BACKGROUND,
            edgeFile = border, tile = true, tileSize = 64, edgeSize = 16,
            insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
        });
    end

    parent:SetBackdropColor(0.0, 0.0, 0.0, 1.0);
    --parent:SetBackdropBorderColor(0.0, 0.0, 0.0, 1.0);
end

function CleanUI_SetArtBackdrop(frame, top, left, bottom, right, rotate)
    local texture = "Interface\\PLAYERACTIONBARALT\\"..CleanUI_GetFrameStyle();

    top = top or 0;
    left = left or 0;
    bottom = bottom or 0;
    right = right or 0;

    local cuiArtFrame = _G[frame:GetName().."ArtBackdrop"];
    if (not cuiArtFrame) then
        cuiArtFrame = CreateFrame("Frame", frame:GetName().."ArtBackdrop", frame, BackdropTemplateMixin and "BackdropTemplate");
    end

    cuiArtFrame:SetParent(frame);
    cuiArtFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", left, top);
    cuiArtFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", right, bottom);
    cuiArtFrame:SetFrameStrata("BACKGROUND");
    
    cuiArtFrame:SetBackdrop( {
            bgFile = nil,
            edgeFile = "Interface\\AddOns\\CleanUI\\skins\\artframe\\borders\\"..CleanUI_GetFrameStyle(),
            tile = true, tileSize = 32, edgeSize = 8.5,
            insets = { left = 10, right = 10, top = 10, bottom = 10 }
        });
    
    if (not cuiArtFrame.background) then
        cuiArtFrame.background = cuiArtFrame:CreateTexture("ARTWORK");
        cuiArtFrame.background:SetPoint("TOPLEFT", cuiArtFrame, "TOPLEFT", 8, -3);
        cuiArtFrame.background:SetPoint("BOTTOMRIGHT", cuiArtFrame, "BOTTOMRIGHT", -8, 3);
        cuiArtFrame.background:SetTexture(texture);
        cuiArtFrame.background:SetTexCoord(0.0, 1.0, 0.1757, 0.3555);

        if (rotate) then
            cuiArtFrame.background:SetTexCoord(0.0, 0.1757, 1.0, 0.1757, 0.0, 0.3555, 1.0, 0.3555);
        end
    end

    return cuiArtFrame;
end

function CleanUI_UpdateAccountData()
    local guid = UnitGUID("player");
    
    local name = UnitName("player");
    
    CleanUIAccountData[guid] = CleanUIAccountData[guid] or {};
    
    CleanUIAccountData[guid].name = UnitName("player");
    CleanUIAccountData[guid].realm = GetRealmName();
    CleanUIAccountData[guid].class = UnitClass("player");
    CleanUIAccountData[guid].race = UnitRace("player");
    CleanUIAccountData[guid].faction = UnitFactionGroup("player");
    CleanUIAccountData[guid].level = UnitLevel("player");
end

function CleanUI_ResetMoney()
    CleanUICurrency["money"] = {};
end

function CleanUI_UpdateMoney()
    CleanUICurrency["money"] = CleanUICurrency["money"] or {};
    
    local money = CleanUICurrency["money"];
    
    money[UnitGUID("player")] = GetMoney();
end

