local MASQUE = LibStub("Masque", true);
local buttongroupActionButtons = nil;
local buttongroupBags = nil;

RANGE_INDICATOR = "";

function CleanUI_InitActionBars()

    CleanUIUserData.scaleActionBars = CleanUIUserData.scaleActionBars or 1.0

    if (MASQUE) then
        buttongroupActionButtons = MASQUE:Group("CleanUI", "Action Buttons");
        buttongroupBags = MASQUE:Group("CleanUI", "Bags");
        
        CleanUI_SkinActionBars();
        CleanUI_SkinBagButtons();
    end

    hooksecurefunc("PetActionBar_Update", CleanUI_InitPetBarArtwork);
    hooksecurefunc(ActionBarActionButtonMixin, "UpdateHotkeys", CleanUI_ActionButton_UpdateHotkeys);
    hooksecurefunc(MainMenuBar, "SetPositionForStatusBars", CleanUI_SetPositionForStatusBars);
    
    CleanUI_UpdateBagButtons();
    CleanUI_UpdateActionButtons();
    
    CleanUI_RemoveMacroNames();
    CleanUI_RemoveArtwork();
    CleanUI_InitActionBarArtwork();
    CleanUI_InitPetBarArtwork();
    
    CleanUI_ScaleActionBars();
    CleanUI_ContainerFrame_UpdateFreeSlots();

    CleanUI_InitActionBarCombopoints();


    local frame = CreateFrame("Frame");
    frame:HookScript("OnUpdate", CleanUI_ActionBar_OnUpdate);
end

local UPDATE_WAIT = 0.5;
local _WaitTimeSinceLast = 0;
function CleanUI_ActionBar_OnUpdate(self, elapsed)
    if (_WaitTimeSinceLast <= 0) then
        CleanUI_UpdateActionButtons(elapsed);
        _WaitTimeSinceLast = UPDATE_WAIT;
    else
        _WaitTimeSinceLast = _WaitTimeSinceLast - elapsed;
    end
end

function CleanUI_UpdateActionButtons(elapsed)
    for i = 1, 12 do
        CleanUI_ActionButton_UpdateHotkeys(_G["ActionButton"..i]);
        CleanUI_ActionButton_UpdateHotkeys(_G["MultiBarBottomLeftButton"..i]);
        CleanUI_ActionButton_UpdateHotkeys(_G["MultiBarBottomRightButton"..i]);
        CleanUI_ActionButton_UpdateHotkeys(_G["MultiBarRightButton"..i]);
        CleanUI_ActionButton_UpdateHotkeys(_G["MultiBarLeftButton"..i]);

        CleanUI_ActionButton_OnUpdate(_G["ActionButton"..i], elapsed);
        CleanUI_ActionButton_OnUpdate(_G["MultiBarBottomLeftButton"..i], elapsed);
        CleanUI_ActionButton_OnUpdate(_G["MultiBarBottomRightButton"..i], elapsed);
        CleanUI_ActionButton_OnUpdate(_G["MultiBarRightButton"..i], elapsed);
        CleanUI_ActionButton_OnUpdate(_G["MultiBarLeftButton"..i], elapsed);
    end

    for i = 1, 12 do
        local button = _G["OverrideActionBarButton"..i];

        if (button) then
            CleanUI_ActionButton_OnUpdate(button, elapsed);
        end
    end
end

function CleanUI_ActionButton_UpdateHotkeys(self)
    local hotkey = self.HotKey;
    local text = hotkey:GetText()

    if (text) then
        text = string.gsub( text, "Ziffernblock ", "Z" )
        hotkey:SetText(text)
    end
end

function CleanUI_InitActionBarCombopoints()
    local maxComboPoints = UnitPowerMax(PlayerFrame.unit, Enum.PowerType.ComboPoints);

    if (maxComboPoints == 0) then
        return;
    end

    MainMenuBar.combopoints = CreateFrame("Frame", "CUIMainMenuBarComboPoints", MainMenuBar);
    MainMenuBar.combopoints:ClearAllPoints();
    MainMenuBar.combopoints:SetSize(10, 10);
    MainMenuBar.combopoints:SetPoint("TOPLEFT", MainMenuBarArtFrame, "TOPLEFT", 0, -11);
    MainMenuBar.combopoints:SetPoint("BOTTOMRIGHT", MainMenuBarArtFrame, "TOPRIGHT", 0, -14);

    local glowSize = 100;

    MainMenuBar.combopointsGlow = UIParent:CreateTexture("CUIMainMenuBarComboPointsGlow", "BACKGROUND");
    MainMenuBar.combopointsGlow:SetPoint("TOPLEFT", CleanUIActionBarArtFrame, "TOPLEFT", -glowSize, glowSize/1.5);
    MainMenuBar.combopointsGlow:SetPoint("BOTTOMRIGHT", CleanUIActionBarArtFrame, "BOTTOMRIGHT", glowSize, 0);
    MainMenuBar.combopointsGlow:SetTexture("Interface\\AddOns\\CleanUI\\skins\\frame\\frame-glow");
    MainMenuBar.combopointsGlow:SetVertexColor(0.0, 1.0, 0.4);
    MainMenuBar.combopointsGlow:SetAlpha(0.0);

    local parent = MainMenuBar.combopoints;

	parent:RegisterEvent("PLAYER_TARGET_CHANGED");
	parent:RegisterEvent("UNIT_POWER_FREQUENT");
	parent:RegisterEvent("UNIT_MAXPOWER");
	parent:RegisterEvent("PLAYER_ENTERING_WORLD");
    
    parent:RegisterEvent("PLAYER_REGEN_DISABLED");
    parent:RegisterEvent("PLAYER_REGEN_ENABLED");
    
    parent:HookScript("OnEvent", CleanUI_ComboPointsOnEvent);

    local last = nil;

    for i=1, maxComboPoints do
        local combo = _G[parent:GetName()..i];
        if (combo == nil) then
            combo = parent:CreateTexture(parent:GetName()..i, "ARTWORK");
            combo:SetWidth((parent:GetWidth() + 2) / maxComboPoints - 3);
            combo:SetHeight(parent:GetHeight());
            combo:SetTexture("Interface\\TargetingFrame\\UI-StatusBar");

            combo:SetAlpha(0.0);
        end

        if (last) then
            combo:ClearAllPoints();
            combo:SetPoint("LEFT", last, "RIGHT", 3, 0);
        else
            combo:SetPoint("TOPLEFT", parent, "TOPLEFT", 0, 0);
        end

        last = combo;
    end
end

local lastComboPoints = 0

function CleanUI_ComboPointsOnEvent(self, event, ...)
    local comboPoints = UnitPower(PlayerFrame.unit, Enum.PowerType.ComboPoints);    
    local maxComboPoints = UnitPowerMax(PlayerFrame.unit, Enum.PowerType.ComboPoints);

    if (lastComboPoints == comboPoints) then
        return;
    end

    lastComboPoints = comboPoints;

    local r = 0;
    local g = 1;
    local b = 0;

    for i=1, maxComboPoints do
        local combo = _G[self:GetName()..i];

        if (comboPoints < maxComboPoints) then
            r = 1.0;
            g = 1.0 / maxComboPoints * i;
            b = 0.2;
        end

        combo:SetVertexColor(r, g, b);

        if (i > comboPoints and combo:GetAlpha() >= 0) then
            UIFrameFadeOut(combo, 0.1, combo:GetAlpha(), 0.0);
        end


        if (i <= comboPoints and combo:GetAlpha() == 0) then
            UIFrameFadeIn(combo, 0.4, combo:GetAlpha(), 1.0);
        end
    end

    if (InCombatLockdown()) then
        if (CleanUISavedVariables.showComboPointsGlow) then
            if (comboPoints == maxComboPoints) then
                UIFrameFadeIn(MainMenuBar.combopointsGlow, 0.4, MainMenuBar.combopointsGlow:GetAlpha(), 0.35);
            else
                UIFrameFadeOut(MainMenuBar.combopointsGlow, 0.1, MainMenuBar.combopointsGlow:GetAlpha(), 0.0);
            end
        end
    else
        MainMenuBar.combopointsGlow:Hide();
    end
end

function CleanUI_SetActionBarsScale(scale)
    if (scale and scale >= 0.5 and scale <= 1.5) then
        CleanUIUserData.scaleActionBars = scale
        CleanUI_ScaleActionBars() 
    else
        DEFAULT_CHAT_FRAME:AddMessage("scale: from 0.5 to 1.5", 1.0, 0, 0, 1, 10);
    end
end

function CleanUI_ScaleActionBars()    
    MainMenuBar:SetScale(CleanUIUserData.scaleActionBars);
    MainMenuBar:SetMovable(true);
    MainMenuBar:SetUserPlaced(true);
end

function CleanUI_SkinActionBars()
    for i = 1, 12 do
        CleanUI_SkinActionButton(_G["ActionButton"..i]);
        CleanUI_SkinActionButton(_G["MultiBarBottomLeftButton"..i]);
        CleanUI_SkinActionButton(_G["MultiBarBottomRightButton"..i]);
        CleanUI_SkinActionButton(_G["MultiBarRightButton"..i]);
        CleanUI_SkinActionButton(_G["MultiBarLeftButton"..i]);
        
        _G["ActionButton"..i]:SetFrameStrata("MEDIUM");
        _G["MultiBarBottomLeftButton"..i]:SetFrameStrata("MEDIUM");
        _G["MultiBarBottomRightButton"..i]:SetFrameStrata("MEDIUM");
        _G["MultiBarRightButton"..i]:SetFrameStrata("MEDIUM");
        _G["MultiBarLeftButton"..i]:SetFrameStrata("MEDIUM");
    end

    for i = 1, 10 do
        CleanUI_SkinActionButton(_G["StanceButton"..i]);
        CleanUI_SkinActionButton(_G["PetActionButton"..i]);
    end
end

function CleanUI_SkinActionButton(button)
    buttongroupActionButtons:AddButton(button);
end

function CleanUI_SkinBagButtons()
    for i=0, 3, 1 do
        buttongroupBags:AddButton(_G["CharacterBag"..i.."Slot"]);
        local iconTexture = _G["CharacterBag"..i.."SlotIconTexture"];
        iconTexture:SetDrawLayer("BACKGROUND");
        iconTexture:SetTexCoord(0.08, 0.92, 0.08, 0.92);
        
        _G["CharacterBag"..i.."Slot"].IconBorder:SetTexture(nil);
        _G["CharacterBag"..i.."Slot"].IconBorder:SetSize(0.1, 0.1);
    end
    
    buttongroupBags:AddButton(MainMenuBarBackpackButton);
    local iconTexture = _G["MainMenuBarBackpackButtonIconTexture"];
    iconTexture:SetDrawLayer("BACKGROUND");
    iconTexture:SetTexCoord(0.08, 0.92, 0.08, 0.92);
end

function CleanUI_RemoveMacroNames()
    for i = 1, 12 do
        _G["ActionButton"..i.."Name"]:Hide();
        _G["MultiBarBottomLeftButton"..i.."Name"]:Hide();
        _G["MultiBarBottomRightButton"..i.."Name"]:Hide();
        _G["MultiBarRightButton"..i.."Name"]:Hide();
        _G["MultiBarLeftButton"..i.."Name"]:Hide();
    end
end

function CleanUI_RemoveArtwork()
    MainMenuBarArtFrame.LeftEndCap:SetTexture();
    MainMenuBarArtFrame.RightEndCap:SetTexture();

    StatusTrackingBarManager.SingleBarLarge:SetTexture();
    StatusTrackingBarManager.SingleBarLargeUpper:SetTexture();
    StatusTrackingBarManager.SingleBarSmall:SetTexture();
    StatusTrackingBarManager.SingleBarSmallUpper:SetTexture();

    for i, bar in ipairs(StatusTrackingBarManager.bars) do
		bar.StatusBar.Background:SetTexture();
	end
end

function CleanUI_InitActionBarArtwork()
    local cuiArtFrame = _G["CleanUIActionBarArtFrame"];
    if (not cuiArtFrame) then
        cuiArtFrame = CreateFrame("Frame", "CleanUIActionBarArtFrame", MainMenuBarArtFrame);
        cuiArtFrame:SetFrameStrata("LOW");
    end

    CleanUI_SetPositionForStatusBars()
    
    local texture = "Interface\\PLAYERACTIONBARALT\\"..CleanUI_GetFrameStyle();
    
    if (not cuiArtFrame.background) then
        cuiArtFrame.background = cuiArtFrame:CreateTexture("ARTWORK");
        cuiArtFrame.background:SetPoint("TOPLEFT", cuiArtFrame, "TOPLEFT", -2, 0);
        cuiArtFrame.background:SetPoint("BOTTOMRIGHT", cuiArtFrame, "BOTTOMRIGHT", 0, 0);
        cuiArtFrame.background:SetTexture(texture);
        cuiArtFrame.background:SetTexCoord(0.0, 1.0, 0.1757, 0.3555);
    end
    
    if (not cuiArtFrame.topborder) then
        cuiArtFrame.topborder = cuiArtFrame:CreateTexture("HIGHLIGHT");
        cuiArtFrame.topborder:SetPoint("TOPLEFT", cuiArtFrame, "TOPLEFT", 0, 0);
        cuiArtFrame.topborder:SetPoint("BOTTOMRIGHT", cuiArtFrame, "TOPRIGHT", 0, -3);
        cuiArtFrame.topborder:SetTexture(texture);
        cuiArtFrame.topborder:SetTexCoord(0.0, 1.0, 0.0898, 0.0977);
    end

    -- gryphons
    
    if (not cuiArtFrame.leftborder) then
        cuiArtFrame.leftborder = cuiArtFrame:CreateTexture("HIGHLIGHT");
        cuiArtFrame.leftborder:SetPoint("TOPLEFT", cuiArtFrame, "TOPLEFT", -145, 45);
        cuiArtFrame.leftborder:SetPoint("BOTTOMRIGHT", cuiArtFrame, "BOTTOMLEFT", 27, -3);
        cuiArtFrame.leftborder:SetTexture(texture);
        cuiArtFrame.leftborder:SetTexCoord(0, 0.32, 0.72, 1);
    end
    
    if (not cuiArtFrame.rightborder) then
        cuiArtFrame.rightborder = cuiArtFrame:CreateTexture("HIGHLIGHT");
        cuiArtFrame.rightborder:SetPoint("TOPLEFT", cuiArtFrame, "TOPRIGHT", -29, 45);
        cuiArtFrame.rightborder:SetPoint("BOTTOMRIGHT", cuiArtFrame, "BOTTOMRIGHT", 143, -3);
        cuiArtFrame.rightborder:SetTexture(texture);
        cuiArtFrame.rightborder:SetTexCoord(0.32, 0, 0.72, 1);
    end
end

function CleanUI_SetPositionForStatusBars()
    local cuiArtFrame = _G["CleanUIActionBarArtFrame"];
    if (cuiArtFrame) then
        local topleft = ActionButton1;
    
        if (MultiBarBottomLeft:IsShown()) then
            topleft = MultiBarBottomLeftButton1;
        end

        cuiArtFrame:SetParent(MainMenuBarArtFrame);
        cuiArtFrame:SetPoint("TOPLEFT", topleft, "TOPLEFT", -8, 5);

        local visibleBars = StatusTrackingBarManager:GetNumberVisibleBars();

        CleanUIActionBarArtFrame:SetPoint("BOTTOMRIGHT", MainMenuBarArtFrame, "BOTTOMRIGHT", 2, -27);

        if ( visibleBars == 2 ) then 
            cuiArtFrame:SetPoint("BOTTOMRIGHT", MainMenuBarArtFrame, "BOTTOMRIGHT", 2, -30);
        elseif ( visibleBars == 1 ) then
            cuiArtFrame:SetPoint("BOTTOMRIGHT", MainMenuBarArtFrame, "BOTTOMRIGHT", 2, -24);
        else 
            cuiArtFrame:SetPoint("BOTTOMRIGHT", MainMenuBarArtFrame, "BOTTOMRIGHT", 2, -10);
        end
    end
end

function CleanUI_InitPetBarArtwork()    
    -- pet bar 
    local firstPetButton = _G["PetActionButton1"];
    local lastPetButton = _G["PetActionButton1"];
    local button;

    for i = 2, 10 do
        button = _G["PetActionButton"..i];
        if (button and button:IsVisible()) then
            lastPetButton = button;
        end
    end
    
    local cuiPetArtFrame = _G["CleanUIPetBarArtFrame"];
    if (not cuiPetArtFrame) then
        cuiPetArtFrame = CreateFrame("Frame", "CleanUIPetBarArtFrame", PetActionBarFrame);
    end

    cuiPetArtFrame:SetParent(PetActionBarFrame);
    cuiPetArtFrame:SetPoint("TOPLEFT", firstPetButton, "TOPLEFT", -12, 5);
    cuiPetArtFrame:SetPoint("BOTTOMRIGHT", lastPetButton, "BOTTOMRIGHT", 12, -3);
    cuiPetArtFrame:SetFrameStrata("BACKGROUND");
    
    local texture = "Interface\\PLAYERACTIONBARALT\\"..CleanUI_GetFrameStyle();
    
    if (not cuiPetArtFrame.background) then
        cuiPetArtFrame.background = cuiPetArtFrame:CreateTexture("ARTWORK");
        cuiPetArtFrame.background:SetPoint("TOPLEFT", cuiPetArtFrame, "TOPLEFT", 0, 0);
        cuiPetArtFrame.background:SetPoint("BOTTOMRIGHT", cuiPetArtFrame, "BOTTOMRIGHT", 0, 0);
        cuiPetArtFrame.background:SetTexture(texture);
        cuiPetArtFrame.background:SetTexCoord(0.0, 1.0, 0.1757, 0.3555);
    end    
    
    if (not cuiPetArtFrame.topborder) then
        cuiPetArtFrame.topborder = cuiPetArtFrame:CreateTexture("HIGHLIGHT");
        cuiPetArtFrame.topborder:SetPoint("TOPLEFT", cuiPetArtFrame, "TOPLEFT", 0, 0);
        cuiPetArtFrame.topborder:SetPoint("BOTTOMRIGHT", cuiPetArtFrame, "TOPRIGHT", 0, -3);
        cuiPetArtFrame.topborder:SetTexture(texture);
        cuiPetArtFrame.topborder:SetTexCoord(0.0, 1.0, 0.0898, 0.0977);
    end
    
    if (not cuiPetArtFrame.leftborder) then
        cuiPetArtFrame.leftborder = cuiPetArtFrame:CreateTexture("HIGHLIGHT");
        cuiPetArtFrame.leftborder:SetPoint("TOPLEFT", cuiPetArtFrame, "TOPLEFT", -2, 3);
        cuiPetArtFrame.leftborder:SetPoint("BOTTOMRIGHT", cuiPetArtFrame, "BOTTOMLEFT", 24, -3);
        cuiPetArtFrame.leftborder:SetTexture(texture);
        cuiPetArtFrame.leftborder:SetTexCoord(0.2402, 0.3027, 0.7910, 0.8750);
    end
    
    if (not cuiPetArtFrame.rightborder) then
        cuiPetArtFrame.rightborder = cuiPetArtFrame:CreateTexture("HIGHLIGHT");
        cuiPetArtFrame.rightborder:SetPoint("TOPLEFT", cuiPetArtFrame, "TOPRIGHT", 2, 3);
        cuiPetArtFrame.rightborder:SetPoint("BOTTOMRIGHT", cuiPetArtFrame, "BOTTOMRIGHT", -24, -3);
        cuiPetArtFrame.rightborder:SetTexture(texture);
        cuiPetArtFrame.rightborder:SetTexCoord(0.3027, 0.2402, 0.7910, 0.8750);
    end
end

function CleanUI_UpdateBagButtons()
    MainMenuBarBackpackButton:SetParent(UIParent);

    CharacterBag0Slot:SetParent(UIParent);
    CharacterBag1Slot:SetParent(UIParent);
    CharacterBag2Slot:SetParent(UIParent);
    CharacterBag3Slot:SetParent(UIParent);

    MainMenuBarBackpackButton:ClearAllPoints();
    MainMenuBarBackpackButton:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -75, 2.2);

    MainMenuBarBackpackButton:SetScale(0.9);
    MainMenuBarBackpackButtonStock:SetScale(1.0);

    CharacterBag0Slot:SetScale(1.2);
    CharacterBag0SlotStock:SetScale(0.75);
    CharacterBag1Slot:SetScale(1.2);
    CharacterBag1SlotStock:SetScale(0.75);
    CharacterBag2Slot:SetScale(1.2);
    CharacterBag2SlotStock:SetScale(0.75);
    CharacterBag3Slot:SetScale(1.2);
    CharacterBag3SlotStock:SetScale(0.75);
    
    CleanUI_InitBagButton(MainMenuBarBackpackButton);
    CleanUI_InitBagButton(CharacterBag0Slot);
    CleanUI_InitBagButton(CharacterBag1Slot);
    CleanUI_InitBagButton(CharacterBag2Slot);
    CleanUI_InitBagButton(CharacterBag3Slot);
end

function CleanUI_InitBagButton(button)
    button:HookScript("OnEnter", CleanUI_BagButton_OnEnter);
    button:HookScript("OnLeave", CleanUI_BagButton_OnLeave);
    CleanUI_BagButton_OnLeave(button)
end

function CleanUI_BagButton_OnEnter(self)
    self:SetAlpha(1.0);
end

function CleanUI_BagButton_OnLeave(self)
    self:SetAlpha(0.7);
end

function CleanUI_ActionButton_OnUpdate(self, elapsed)
    CleanUI_ActionButtonDeBuffs_OnUpdate(self, elapsed);

    local valid = IsActionInRange(self.action);
    
    if (valid ~= false) then
        self.icon:SetDesaturated(nil);
        self.icon:SetVertexColor(1.0, 1.0, 1.0);
    else
        self.icon:SetDesaturated(1);
        self.icon:SetVertexColor(1.0, 0.0, 0.0);
    end
    
    if (valid ~= false) then
        local isUsable, notEnoughMana = IsUsableAction(self.action);
        
        if (notEnoughMana) then
            local powerType, powerTypeString = UnitPowerType("player");
            local color = PowerBarColor[powerTypeString];
            
            self.icon:SetDesaturated(1);
            --self.icon:SetVertexColor(0.1, 0.25, 1.0);
            self.icon:SetVertexColor(color.r/2, color.g/2, color.b/2);
        elseif (not isUsable) then
            self.icon:SetDesaturated(1);
            self.icon:SetVertexColor(0.0, 0.4, 0.2);
        else
            self.icon:SetDesaturated(nil);
            self.icon:SetVertexColor(1.0, 1.0, 1.0);
        end
    end
end