
function CleanUI_InitCharacterFrame()

    CleanUI_CharacterFrameHookEvents();
    CleanUI_RegisterCharacterFrameEvents();
end

function CleanUI_RegisterCharacterFrameEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");
    --frame:RegisterEvent("MERCHANT_SHOW");
    --frame:RegisterEvent("BANKFRAME_OPENED");
    --frame:RegisterEvent("BAG_UPDATE");

    frame:HookScript("OnEvent", CleanUI_CharacterFrameOnEvent);
end

function CleanUI_CharacterFrameOnEvent(self, event, ...)

    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_InspectUI" ) then
            hooksecurefunc("InspectPaperDollItemSlotButton_Update", CleanUI_InspectPaperDollItemSlotButton_Update);
        end
    end

    --if (event == "MERCHANT_SHOW") then
        --CleanUI_MerchantShow();
    --end
end

function CleanUI_CharacterFrameHookEvents()
    hooksecurefunc("ReputationFrame_Update", CleanUI_ReputationFrame_Update);
    hooksecurefunc("PaperDollItemSlotButton_Update", CleanUI_PaperDollItemSlotButton_Update);

    for i=1, 15, 1 do
        local bar = _G["ReputationBar"..i];
        bar:HookScript("OnEnter", CleanUI_ReputationBarOnEnter);
        bar:HookScript("OnLeave", CleanUI_ReputationBarOnLeave);
    end
end

function CleanUI_ReputationBarOnEnter(self)
end

function CleanUI_ReputationBarOnLeave(self)
    local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain, canBeLFGBonus = GetFactionInfo(self.index);
    
    if ( factionID and C_Reputation.IsFactionParagon(factionID) ) then
        _G[self:GetName().."ReputationBarFactionStanding"]:SetText("Paragon");
    end
end

function CleanUI_ReputationFrame_Update(showLFGPulse)
    local factionOffset = FauxScrollFrame_GetOffset(ReputationListScrollFrame);
    local numFactions = GetNumFactions();
    
    for i=1, NUM_FACTIONS_DISPLAYED, 1 do
        local factionIndex = factionOffset + i;
        if ( factionIndex <= numFactions ) then
            local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain, canBeLFGBonus = GetFactionInfo(factionIndex);
        
            local factionRow = _G["ReputationBar"..i];
            local factionBar = _G["ReputationBar"..i.."ReputationBar"];

            if ( factionID and C_Reputation.IsFactionParagon(factionID) ) then
                factionBar:SetStatusBarColor(0.3, 0.4, 1.0);
                
                local currentValue, threshold, rewardQuestID, hasRewardPending = C_Reputation.GetFactionParagonInfo(factionID);

                --Normalize Values
                local max = threshold;
                local value = currentValue;
                
                while (value > max) do
                    value = value - max;
                end
                
                factionBar:SetMinMaxValues(0, max);
                factionBar:SetValue(value);
                
                factionRow.rolloverText = value.."/"..max;
                
                -- Text
                local factionStanding = _G["ReputationBar"..i.."ReputationBarFactionStanding"];
                factionStanding:SetText("Paragon");
            elseif (standingID == 8) then
                factionBar:SetStatusBarColor(0.6, 0.8, 1.0);
            end
        end
    end
end

function CleanUI_PaperDollItemSlotButton_Update(self)
    CleanUI_ItemSlotButton_Update(self, "player");
end

function CleanUI_InspectPaperDollItemSlotButton_Update(self)
    local unit = InspectFrame.unit;
    CleanUI_ItemSlotButton_Update(self, unit);
end

function CleanUI_ItemSlotButton_Update(self, unit)
    if (self:GetName() ~= nil and string.find(self:GetName(), "CharacterBag") ~= nil) then
        return;
    end

    -- create label
    local itemLevelLabel = _G[self:GetName().."CItemLevel"];
    if (not itemLevelLabel) then
        itemLevelLabel = self:CreateFontString(self:GetName().."CItemLevel", "ARTWORK");
        itemLevelLabel:SetFontObject("GameFontNormalSmall");
        itemLevelLabel:SetSize(self:GetWidth(), 10);
        itemLevelLabel:ClearAllPoints();
        itemLevelLabel:SetPoint("TOPRIGHT", -2, -2);
        itemLevelLabel:SetJustifyH("RIGHT");
        itemLevelLabel:SetText("");
    end
    
    local itemLink = GetInventoryItemLink(unit, self:GetID());
    
    -- no item link, no label
    if (itemLink == nil) then
        itemLevelLabel:Hide();
        return;
    end
    
    -- show
    itemLevelLabel:Show();
    
    -- calculate level
    local effectiveILvl, isPreview, baseILvl = GetDetailedItemLevelInfo(itemLink);
    
    -- colorize
    local quality = GetInventoryItemQuality(unit, self:GetID());
    local color = ITEM_QUALITY_COLORS[quality];
    local hexcolor = cui_rgb2hex(color.r, color.g, color.b);
    
    -- set text
    itemLevelLabel:SetText("|cff"..hexcolor..effectiveILvl.."|r");
end

-- OrderHallFix

local t = {
    [23] = function() return select(4, GetMapInfo()) and 1007 end, -- Paladin, Sanctum of Light; Eastern Plaguelands
    [1040] = function() return 1007 end, -- Priest, Netherlight Temple; Azeroth
    [1044] = function() return 1007 end, -- Monk, Temple of Five Dawns; none
    [1048] = function() return 1007 end, -- Druid, Emerald Dreamway; none
    [1052] = function() return GetCurrentMapDungeonLevel() > 1 and 1007 end, -- Demon Hunter, Fel Hammer; Mardum
    [1088] = function() return GetCurrentMapDungeonLevel() == 3 and 1033 end, -- Nighthold -> Suramar
}

local OnClick = WorldMapZoomOutButton_OnClick

function WorldMapZoomOutButton_OnClick()
    local id = t[GetCurrentMapAreaID()];
    local out = id and id();
    
    if out then
        SetMapByID(out);
    else
        OnClick();
    end
end

