local battlefieldMapFrameDone = false;

local partyMembersOnTaxi = {};

function CleanUI_InitMap()
    CleanUIUserData.scaleMap = CleanUIUserData.scaleMap or 1.0;

    CleanUI_MapHookEvents();
    CleanUI_RegisterMapEvents();
    
    CleanUI_ChangeMap();
    CleanUI_ChangeBountyFrame();

    if (BattlefieldMapFrame) then
        CleanUI_ChangeBattlefieldMap();
        battlefieldMapFrameDone = true
    end
end

local function getBountyFrame()

    local bountyFrame = nil

    --WorldMapBountyBoardTemplate
    for i, child in ipairs({WorldMapFrame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then
            if (child.BountyName ~= nil) then
                return child;
            end
        end
    end

    return nil;
end

function CleanUI_ChangeBountyFrame()
    local bountyFrame = getBountyFrame();
    --bountyFrame:ClearAllPoints();
    --bountyFrame:SetPoint("TOPLEFT", WorldMapFrame, "BOTTOMLEFT", 0, 0)
    bountyFrame:SetScale(0.9)
end

function CleanUI_RegisterMapEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");
    frame:RegisterEvent("CHAT_MSG_ADDON");
    frame:RegisterEvent("UNIT_EXITED_VEHICLE");

    frame:HookScript("OnEvent", CleanUI_MapOnEvent);

    WorldMapFrame:HookScript("OnUpdate", CleanUI_WorldMapFrame_OnUpdate);
    hooksecurefunc("TakeTaxiNode", CleanUI_TakeTaxiNode);
end

function CleanUI_MapOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_BattlefieldMap" ) then
            if (not battlefieldMapFrameDone) then
                CleanUI_ChangeBattlefieldMap();
            end
        end
        if ( addonName == "Blizzard_FlightMap" ) then
            hooksecurefunc(FlightMap_WorldQuestPinMixin, "RefreshVisuals", CleanUI_WorldQuestPinMixin_RefreshVisuals);
            hooksecurefunc(MapCanvasMixin, "OnCanvasScaleChanged", CleanUI_UpdateFlightPathMap);
            hooksecurefunc("MainMenuBarVehicleLeaveButton_Update", CleanUI_MainMenuBarVehicleLeaveButton_Update);
            FlightMapFrame:HookScript("OnShow", CleanUI_UpdateFlightPathMap);
            FlightMapFrame:HookScript("OnUpdate", CleanUI_UpdateFlightPathMap);
        end
    end

    if (event == "CHAT_MSG_ADDON") then
        local prefix, message, type, sender = ...;
        if (prefix == "CUI_SEND_FP") then
            CleanUI_ReceiveFlightPathInfo(message);
        end
    end

    if (event == "UNIT_EXITED_VEHICLE") then
        local unitToken = ...;
        CleanUI_UnitExitedVehicle(unitToken);
    end
end

function CleanUI_SetMapScale(scale)
    if (scale and scale >= 0.5 and scale <= 1) then
        CleanUIUserData.scaleMap = scale;
    else
        DEFAULT_CHAT_FRAME:AddMessage("scale: from 0.5 to 1.0", 1.0, 0, 0, 1, 10);
    end
end

function CleanUI_TakeTaxiNode(id)
    if (FlightMapFrame and FlightMapFrame:IsVisible()) then
        if ( UnitInParty("player") ) then
            local mapID = FlightMapFrame:GetMapID();
            local taxiNodes = C_TaxiMap.GetAllTaxiNodes(mapID);
            for i, taxiNodeData in ipairs(taxiNodes) do
                if taxiNodeData.slotIndex == id then
                    local x, y = taxiNodeData.position:GetXY();
                    local message = "IN;"..taxiNodeData.name..";"..taxiNodeData.slotIndex..";"..UnitName("player")..";"..x..";"..y..";"..mapID;
                    local success = C_ChatInfo.SendAddonMessage("CUI_SEND_FP", message, "PARTY");
                end
            end
        end
    elseif (TaxiFrame and  TaxiFrame:IsVisible()) then
        local x, y = TaxiNodePosition(id);
        local name = TaxiNodeName(id);
        local type = TaxiNodeGetType(id);

        if ( type == "REACHABLE" ) then
            local mapID = GetTaxiMapID();
            local message = "IN;"..name..";"..id..";"..UnitName("player")..";"..x..";"..y..";"..mapID;
            local success = C_ChatInfo.SendAddonMessage("CUI_SEND_FP", message, "PARTY");
        end
    end
end

function CleanUI_MainMenuBarVehicleLeaveButton_Update()
    if ( CanExitVehicle() and ActionBarController_GetCurrentActionBarState() == LE_ACTIONBAR_STATE_MAIN ) then
        -- in vehicle
    else
        CleanUI_UnitExitedVehicle("player");
    end
end

function CleanUI_UnitExitedVehicle(unitToken)
    if (unitToken == "player") then
        local message = "OUT;"..UnitName(unitToken);
        local success = C_ChatInfo.SendAddonMessage("CUI_SEND_FP", message, "PARTY");
    end
end

function CleanUI_ReceiveFlightPathInfo(message)
    local type, name, slotIndex, sender, str_x, str_y, str_mapid = strsplit(";", message, 7);
    local x = tonumber(str_x);
    local y = tonumber(str_y);
    local mapID = tonumber(str_mapid);


    if (type == "IN") then
        if (UnitName("player") ~= sender) then
            DEFAULT_CHAT_FRAME:AddMessage(sender.." fliegt nach "..name..".", 0.7, 0.8, 1.0, 53, 10);
            partyMembersOnTaxi[sender] = {slotIndex=slotIndex, name=name, sender=sender, x=x, y=y, y=y, mapID=mapID};
            CleanUI_UpdateFlightPathMap();
        end
    else
        local copyData = {};
        for foundName, data in pairs(partyMembersOnTaxi) do
            if (name ~= foundName) then
                copyData[foundName] = data;
            end
        end
        partyMembersOnTaxi = copyData;
        CleanUI_UpdateFlightPathMap();
    end
end

function CleanUI_UpdateFlightPathMap()
    local frame, x, y;

    local index = 1;

    if (FlightMapFrame and FlightMapFrame:IsVisible()) then
        if (FlightMapFrame:GetCanvasZoomPercent() == 0) then
            local mapID = FlightMapFrame:GetMapID();

            for name, data in pairs(partyMembersOnTaxi) do
                if (mapID == data.mapID) then
                    frame = CleanUI_GetOrCreateTargetMarker("FlightMapFrame", FlightMapFrame.ScrollContainer, index);

                    x = FlightMapFrame.ScrollContainer:GetWidth() * data.x;
                    y = FlightMapFrame.ScrollContainer:GetHeight() * -data.y;

                    frame:ClearAllPoints();
                    frame:SetPoint("CENTER", FlightMapFrame.ScrollContainer, "TOPLEFT", x, y);
                    frame:Show();

                    frame.infotext:SetText(name);

                    index = index + 1;
                end
            end
        end

        for i=index, 5 do
            frame = CleanUI_GetOrCreateTargetMarker("FlightMapFrame", FlightMapFrame.ScrollContainer, i);
            frame:Hide();
        end
    end
    
    if (TaxiFrame and TaxiFrame:IsVisible()) then
        local mapID = GetTaxiMapID();

        for name, data in pairs(partyMembersOnTaxi) do
            if (mapID == data.mapID) then
                frame = CleanUI_GetOrCreateTargetMarker("TaxiRouteMap", TaxiRouteMap, index);
    
                x = TaxiRouteMap:GetWidth() * data.x;
                y = TaxiRouteMap:GetHeight() * -data.y;
    
                frame:ClearAllPoints();
                frame:SetPoint("CENTER", TaxiRouteMap, "TOPLEFT", x, y);
                frame:Show();
    
                frame.infotext:SetText(name);
    
                index = index + 1;
            end
        end

        for i=index, 5 do
            frame = CleanUI_GetOrCreateTargetMarker("TaxiRouteMap", TaxiRouteMap, i);
            frame:Hide();
        end
    end
end

function CleanUI_GetOrCreateTargetMarker(namePrefix, parent, index)
    local name = "CUI_"..namePrefix.."_TargetMarker"..index;

    local frame = _G[name];
    if (not frame) then
        frame = CreateFrame("Frame", name, parent);
        frame:SetParent(parent);
        frame:SetFrameStrata("HIGH");
        frame:SetSize(40, 40);

        frame.background = frame:CreateTexture("ARTWORK");
        frame.background:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0);
        frame.background:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, 0);
        frame.background:SetTexture("Interface\\Artifacts\\Artifacts-PerkRing-Final-Mask");

        frame.infotext = frame:CreateFontString(nil, "ARTWORK");
        frame.infotext:SetFontObject("GameFontNormalSmall");
        frame.infotext:SetWidth(frame:GetWidth());
        frame.infotext:SetHeight(14);
        frame.infotext:ClearAllPoints();
        frame.infotext:SetPoint("TOP", frame, "BOTTOM", 0, 8);
        frame.infotext:SetJustifyH("CENTER");
    end

    return frame;
end

function CleanUI_MapHookEvents()    
    hooksecurefunc("WorldMap_AddQuestTimeToTooltip", function(questID)
        local timeLeftMinutes = C_TaskQuest.GetQuestTimeLeftMinutes(questID)
        if timeLeftMinutes then
            local timeLeftSeconds = timeLeftMinutes * 60;
           _G["GameTooltipTextLeft"..GameTooltip:NumLines()]:SetText(BONUS_OBJECTIVE_TIME_LEFT:format(SecondsToTime(timeLeftSeconds)));
        end
    end)
    
    WorldMapFrame:HookScript("OnShow", CleanUI_WorldMapFrame_OnShow);

    hooksecurefunc(WorldMap_WorldQuestPinMixin, "RefreshVisuals", CleanUI_WorldQuestPinMixin_RefreshVisuals);
end

function CleanUI_WorldQuestPinMixin_RefreshVisuals(self)
    local inset = 0;
    local border = -9;
    local epic_border = -15;
    
    if (not self.cui_texture) then
        self.cui_texture = self:CreateTexture("ARTWORK");
        self.cui_texture:SetPoint("TOPLEFT", self, "TOPLEFT", inset, -inset);
        self.cui_texture:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -inset, inset);
        self.cui_texture:SetMask("Interface\\CharacterFrame\\TempPortraitAlphaMask");
    end

    if (not self.cui_faction) then
        self.cui_faction = self:CreateTexture("ARTWORK");
        self.cui_faction:SetSize(65, 65);
        self.cui_faction:SetPoint("CENTER", self, "CENTER", 25, 25);
        self.cui_faction:Show();
        self.cui_faction:SetDrawLayer("ARTWORK", 7);
    end

    self.BountyRing:SetPoint("CENTER", self, "CENTER", 0, -2.5);
    self.BountyRing:SetSize(90, 90);
    self.BountyRing:SetDesaturated(1);
    self.BountyRing:SetVertexColor(1.0, 0.2, 0.3);

    if (not self.cui_border) then
        self.cui_border = self:CreateTexture("ARTWORK");
        self.cui_border:SetPoint("TOPLEFT", self, "TOPLEFT", border, -border);
        self.cui_border:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -border, border);
        self.cui_border:SetTexture("Interface\\CharacterFrame\\TotemBorder");
        self.cui_border:SetDrawLayer("ARTWORK", 6);
    end
    
    if (not self.cui_infotext) then
        self.cui_infotext = self:CreateFontString(nil, "ARTWORK");
        self.cui_infotext:SetFontObject("GameFontNormalHuge");
        self.cui_infotext:SetWidth(self:GetWidth());
        self.cui_infotext:SetHeight(14);
        self.cui_infotext:ClearAllPoints();
        self.cui_infotext:SetPoint("TOP", self, "BOTTOM", 0, -2);
        self.cui_infotext:SetJustifyH("CENTER");
    end
    
    if (not self.cui_questname) then
        self.cui_questname = self:CreateFontString(nil, "ARTWORK");
        self.cui_questname:SetFontObject("GameFontHighlightHuge2");
        self.cui_questname:SetHeight(14);
        self.cui_questname:ClearAllPoints();
        self.cui_questname:SetPoint("LEFT", self, "RIGHT", 10, 0);
        self.cui_questname:SetJustifyH("CENTER");
        self.cui_questname:Hide();
    end

    if (not self.cui_factionMarker) then
        self.cui_factionMarker = self:CreateTexture("ARTWORK");
        self.cui_factionMarker:SetSize(27, 27);
        self.cui_factionMarker:SetPoint("CENTER", self, "TOP", 0, 0);
        self.cui_factionMarker:SetTexture("Interface\\RaidFrame\\ReadyCheck-NotReady");
        self.cui_factionMarker:SetDrawLayer("ARTWORK", 7);
    end

    if (not self.cui_petBattleMarker) then
        self.cui_petBattleMarker = self:CreateTexture("ARTWORK");
        self.cui_petBattleMarker:SetSize(27, 27);
        self.cui_petBattleMarker:SetPoint("CENTER", self, "TOPRIGHT", -6, -6);
        self.cui_petBattleMarker:SetTexture("Interface\\Icons\\Tracking_WildPet");
        self.cui_petBattleMarker:SetDrawLayer("ARTWORK", 7);
    end
    
    self.cui_infotext:SetText("");

    if (self.questID) then
        local tagInfo = C_QuestLog.GetQuestTagInfo(self.questID);
        
        local numQuestCurrencies = GetNumQuestLogRewardCurrencies(self.questID);
        local numQuestLogRewards = GetNumQuestLogRewards(self.questID);
        local money = GetQuestLogRewardMoney(self.questID);

        if (numQuestCurrencies >= 1) then
            local name, texture, count, currencyID = GetQuestLogRewardCurrencyInfo(1, self.questID);

            self.cui_texture:SetTexture(texture);
            self.cui_texture:Show();
            self.cui_border:Show();
            self.Texture:Hide();

            local itemRarity = select(8, C_CurrencyInfo.GetCurrencyInfo(currencyID));

            if (itemRarity) then
                local rarityColor = ITEM_QUALITY_COLORS[itemRarity];
                self.cui_border:SetDesaturated(1);
                self.cui_border:SetVertexColor(rarityColor.r, rarityColor.g, rarityColor.b);
            else
                self.cui_border:SetDesaturated(nil);
                self.cui_border:SetVertexColor(1.0, 1.0, 1.0);
            end

            if (count > 1) then
                self.cui_infotext:SetText(""..count);
                self.cui_infotext:Show();
            else
                self.cui_infotext:Hide();
            end
        elseif numQuestLogRewards >= 1 then
            local name, texture, count, quality, isUsable, itemId = GetQuestLogRewardInfo(1, self.questID);
            self.cui_texture:SetTexture(texture);
            self.cui_texture:Show();
            self.cui_border:Show();
            self.Texture:Hide();

            if (quality) then
                local r, g, b = GetItemQualityColor(quality);
                self.cui_border:SetDesaturated(1);
                self.cui_border:SetVertexColor(r, g, b);
            else
                self.cui_border:SetDesaturated(nil);
                self.cui_border:SetVertexColor(1.0, 1.0, 1.0);
            end
        elseif money > 0 then
            self.cui_texture:SetTexture("Interface\\Icons\\inv_misc_coin_01");
            self.cui_texture:Show();
            self.cui_border:Show();
            self.cui_infotext:Hide();

            self.Texture:Hide();

            self.cui_border:SetDesaturated(nil);
            self.cui_border:SetVertexColor(1.0, 1.0, 1.0);
        else
            self.cui_texture:Hide();
            self.cui_border:Hide();
            self.cui_infotext:Hide();
            self.Texture:Show();

            self.cui_border:SetDesaturated(nil);
            self.cui_border:SetVertexColor(1.0, 1.0, 1.0);
        end

        if ( tagInfo.worldQuestType == Enum.QuestTagType.PetBattle ) then
            self.cui_petBattleMarker:Show();
        else
            self.cui_petBattleMarker:Hide();
        end

        local questName, factionID = C_TaskQuest.GetQuestInfoByQuestID(self.questID);

        if (IsAltKeyDown()) then
            self.cui_questname:Show();
            self.cui_questname:SetText(questName);
        else
            self.cui_questname:Hide();
        end

        if (factionID) then
            local factionName, _, standingID = GetFactionInfoByID(factionID);

            if (standingID == 8) then
                self.cui_factionMarker:Show();
            else
                self.cui_factionMarker:Hide();
            end
            self.cui_factionMarker:Hide();
        end
    end


    if (self.worldQuestType == Enum.QuestTagType.Invasion or self.worldQuestType == Enum.QuestTagType.FactionAssault) then
        local factionTag = UnitFactionGroup("player");
		if factionTag == "Alliance" then
            self.cui_faction:SetTexture("Interface\\Timer\\Alliance-Logo");
		elseif factionTag == "Horde" then
            self.cui_faction:SetTexture("Interface\\Timer\\Horde-Logo");
        else -- "Neutral"
            self.cui_faction:SetTexture("Interface\\Timer\\Panda-Logo");
        end
        
        self.cui_faction:Show();
    else
        self.cui_faction:Hide();
    end
end

local mouseOver = nil;

function CleanUI_WorldMapFrame_OnShow()
    PlayerMovementFrameFader.AddDeferredFrame(WorldMapFrame, .4, 1.0, .5, function() return GetCVarBool("mapFade") and not WorldMapFrame:IsMouseOver() end);
    mouseOver = nil;
    CleanUI_WorldMapFrame_OnUpdate();
end

function CleanUI_WorldMapFrame_OnUpdate()
    if (WorldMapFrame:IsMouseOver() == mouseOver) then
        return;
    end

    if (WorldMapFrame:IsMouseOver()) then
        WorldMapFrame.BorderFrame:Show();
        WorldMapFrame.NavBar:Show();
        WorldMapFrameBg:Show();

        WorldMapFrame:SetScale(1.0);
    
        --CleanUI_RemoveBackdrop(WorldMapFrame);
    else
        if ( QuestMapFrame:IsShown() ) then
            return;
        end
    
        WorldMapFrame.BorderFrame:Hide();
        WorldMapFrame.NavBar:Hide();
        WorldMapFrameBg:Hide();

        WorldMapFrame:SetScale(CleanUIUserData.scaleMap);
    
        --CleanUI_SetBackdrop(WorldMapFrame, true, nil, -63, -1, -1, 0);
    end
    
    --WorldMapFrame:ResetZoom();
    WorldMapFrame:OnCanvasScaleChanged();
    --WorldMapFrame.ScrollContainer:RefreshCanvasScale();

    mouseOver = WorldMapFrame:IsMouseOver();
end

function CleanUI_ChangeMap()
    -- Koordinaten
    local coords = WorldMapFrame:CreateFontString("CUIWorldMapFrameCoords", "OVERLAY");
    coords:SetFontObject("GameFontNormalSmall");
    coords:SetWidth(WorldMapFrame.ScrollContainer:GetWidth());
    coords:SetHeight(20);
    coords:ClearAllPoints();
    coords:SetPoint("BOTTOMLEFT", WorldMapFrame.ScrollContainer, "BOTTOMLEFT", 0, -20);
    coords:SetPoint("BOTTOMRIGHT", WorldMapFrame.ScrollContainer, "BOTTOMRIGHT", 0, -20);
    coords:SetJustifyH("CENTER");
    coords:SetText("Hallo");

    local refFrame = CreateFrame("Frame", "CUI_WorldMapFrameCoordsFrame", WorldMapFrame);
    refFrame:HookScript("OnUpdate", CleanUI_UpdateMapCoords);
end

function CleanUI_UpdateMapCoords()
    if (not WorldMapFrame:IsVisible()) then
        return;
    end

    -- player
    local px, py = CleanUI_GetPlayerMapPosition();
    
    -- cursor
    local adjustedX, adjustedY = CleanUI_GetCursorPosition()

    local output = UnitName("player")..": "..px.."|cffffffff/|r"..py;

    if (adjustedX and adjustedY) then
        output = output.." -- Cursor: "..adjustedX.."|cffffffff/|r"..adjustedY;
    end

    CUIWorldMapFrameCoords:SetText(output);
end

-- TODO CleanUI_Bounties
function CleanUI_Bounties()
    local mapID = C_Map.GetBestMapForUnit("player");

    local bounties, displayLocation, lockedQuestID = GetQuestBountyInfoForMapID(mapID);
    for bountyIndex, bounty in ipairs(bounties) do
    cui_debug("questID", bounty.questID)
    cui_debug("icon", bounty.icon)
    cui_debug("bounty", bounty)

    --IsQuestComplete(bounty.questID)
    --IsQuestCriteriaForBounty(wg.questID, bounty.questID));
    end
end

function CleanUI_GetPlayerMapPosition()
    local mapID = C_Map.GetBestMapForUnit("player");

    if (mapID and C_Map.GetPlayerMapPosition(mapID, "player")) then
        local x, y = C_Map.GetPlayerMapPosition(mapID, "player"):GetXY();
        
        x = x or 0;
        y = y or 0;
    
        x = x * 100;
        y = y * 100;
    
        return "|cffffffff"..string.format("%.1f", x).."|r", "|cffffffff"..string.format("%.1f", y).."|r";
    end

    return "|cffffffff0|r", "|cffffffff0|r";
end

function CleanUI_GetCursorPosition()
    local adjustedX, adjustedY = WorldMapFrame:GetNormalizedCursorPosition();

    adjustedX = adjustedX * 100;
    adjustedY = adjustedY * 100;

    if (adjustedX == 0 or adjustedY == 0 or adjustedX >= 100 or adjustedY >= 100) then
        return nil, nil, 0, 0;
    end

    return "|cffffffff"..string.format("%.1f", adjustedX).."|r", "|cffffffff"..string.format("%.1f", adjustedY).."|r", adjustedX, adjustedY;
end

function CleanUI_ChangeBattlefieldMap()
    BattlefieldMapFrame.BorderFrame:Hide();
    CleanUI_SetBackdrop(BattlefieldMapFrame, false, nil, 5, -3, 0, 1);
end