function CleanUI_InitMiniMap()
    CleanUIUserData.scaleMiniMap = CleanUIUserData.scaleMiniMap or 1.1;

    CleanUI_InitLocationBar();

    CleanUI_MiniMapHookEvents();
    CleanUI_RegisterMiniMapEvents();
    
    CleanUI_ChangeMiniMap();
    CleanUI_ScaleMiniMap();
    
    CleanUI_ChangeMiniMapArt();

    CleanUI_CollectMinimapButtons();
end

function CleanUI_GarrisonLandingPageMinimapButton_UpdateIcon()
    GarrisonLandingPageMinimapButton:SetScale(1);
    GarrisonLandingPageMinimapButton:ClearAllPoints();
    GarrisonLandingPageMinimapButton:SetPoint("TOP", MinimapCluster, "BOTTOMRIGHT", -25, 45);
end

function CleanUI_RegisterMiniMapEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");

    frame:RegisterEvent("ZONE_CHANGED");
    frame:RegisterEvent("ZONE_CHANGED_INDOORS");
    frame:RegisterEvent("ZONE_CHANGED_NEW_AREA");

    frame:HookScript("OnEvent", CleanUI_MiniMapOnEvent);
    frame:HookScript("OnUpdate", CleanUI_MiniMapOnUpdate);  

    hooksecurefunc("GarrisonLandingPageMinimapButton_UpdateIcon", CleanUI_GarrisonLandingPageMinimapButton_UpdateIcon);
    CleanUI_GarrisonLandingPageMinimapButton_UpdateIcon();
end

function CleanUI_MiniMapOnEvent(self, event, ...)

    if (event == "ADDON_LOADED") then
        local addonName = ...;
        CleanUI_CollectMinimapButtons();
    end

    if (event == "ZONE_CHANGED" or event == "ZONE_CHANGED_INDOORS" or event == "ZONE_CHANGED_NEW_AREA") then
        CleanUI_Minimap_UpdateLocationBar();
    end

    --if (event == "MERCHANT_SHOW") then
        --CleanUI_MerchantShow();
    --end
end

local UPDATE_WAIT = 0.5;
local _WaitTimeSinceLast = 0;
function CleanUI_MiniMapOnUpdate(self, elapsed)
    if (_WaitTimeSinceLast <= 0) then
        CleanUI_UpdateMiniMapCoords();
        _WaitTimeSinceLast = UPDATE_WAIT;
    else
        _WaitTimeSinceLast = _WaitTimeSinceLast - elapsed;
    end
end

function CleanUI_MiniMapHookEvents()
end

function CleanUI_SetMiniMapScale(scale)
    if (scale and scale >= 0.5 and scale <= 2) then
        CleanUIUserData.scaleMiniMap = scale;
        CleanUI_ScaleMiniMap();
    else
        DEFAULT_CHAT_FRAME:AddMessage("scale: from 0.5 to 2.0", 1.0, 0, 0, 1, 10);
    end
end

function CleanUI_ScaleMiniMap()    
    MinimapCluster:SetScale(CleanUIUserData.scaleMiniMap);
end

function CleanUI_ChangeMiniMap()
    -- Koordinaten
    local coords = Minimap:CreateFontString("CUIWorldMiniMapFrameCoords", "ARTWORK");
    coords:SetFontObject("CuiFont_ExtraSmall");
    coords:SetWidth(Minimap:GetWidth());
    coords:SetHeight(10);
    coords:ClearAllPoints();
    coords:SetPoint("BOTTOMLEFT", Minimap, "BOTTOMLEFT", 0, 0);
    coords:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMRIGHT", 0, 0);
    coords:SetJustifyH("CENTER");
    coords:SetText("Hallo");
end

function CleanUI_ChangeMiniMapArt()
    MinimapCluster:ClearAllPoints();
    MinimapCluster:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -3, -3);

    Minimap:SetMaskTexture("Interface\\AddOns\\CleanUI\\skins\\minimap\\masktexture");
    MinimapBorder:Hide();
    MinimapBorderTop:Hide();
    
    Minimap:ClearAllPoints();
    Minimap:SetPoint("TOP", MinimapCluster, "TOP", -5, -4);

    -- lift artframe edges
    local cuiArtFrame = CleanUI_SetArtBackdrop(MinimapCluster, -1, 13, 45, -23);

    -- no north
    MinimapNorthTag:SetTexture();
    
    -- zone text

    MinimapZoneTextButton:ClearAllPoints();
    MinimapZoneTextButton:SetPoint("TOPLEFT", MinimapCluster, "TOPLEFT", 19, -8);
    MinimapZoneTextButton:SetPoint("BOTTOMRIGHT", MinimapCluster, "TOPRIGHT", -29, -25);
    MinimapZoneTextButton:Hide();
    
    -- world map button

    MiniMapWorldMapButton:Hide();
    
    --[[
    MiniMapWorldMapButton:SetNormalTexture("Interface\\worldmap\\UI-World-Icon");
    MiniMapWorldMapButton:SetPushedTexture("Interface\\worldmap\\UI-World-Icon");
    MiniMapWorldMapButton:SetHighlightTexture("Interface\\worldmap\\UI-World-Icon");
    
    MiniMapWorldMapButton:GetNormalTexture():SetTexCoord(0, 1, 0, 1);
    MiniMapWorldMapButton:GetPushedTexture():SetTexCoord(0, 1, 0, 1);
    MiniMapWorldMapButton:GetHighlightTexture():SetTexCoord(0, 1, 0, 1);
    
    MiniMapWorldMapButton:SetSize(23, 23);
    MiniMapWorldMapButton:SetPoint("TOPRIGHT", 0, 20);
    ]]

    -- game time frame
    
    GameTimeFrame:SetSize(30, 30);
    GameTimeFrame:SetScale(0.75);
    
    GameTimeFrame:ClearAllPoints();
    GameTimeFrame:SetPoint("TOP", MiniMapWorldMapButton, "TOPRIGHT", -9, -4);

    GameTimeFrame:SetNormalFontObject("GameFontBlackSmall");

    if (not GameTimeFrame.cui_border) then
        GameTimeFrame.cui_border = GameTimeFrame:CreateTexture("ARTWORK");
        GameTimeFrame.cui_border:SetPoint("TOPLEFT", GameTimeFrame, "TOPLEFT", -2, 0);
        GameTimeFrame.cui_border:SetPoint("BOTTOMRIGHT", GameTimeFrame, "BOTTOMRIGHT", 22, -22);
        GameTimeFrame.cui_border:SetTexture("Interface\\Minimap\\MiniMap-TrackingBorder");
    end
    
    -- tracking button
    
    MiniMapTracking:SetSize(30, 30);
    MiniMapTracking:SetScale(0.75);
    
    MiniMapTracking:ClearAllPoints();
    MiniMapTracking:SetPoint("TOP", GameTimeFrame, "BOTTOM", -2, -2);
    
    -- mail frame
    
    MiniMapMailFrame:SetSize(34, 34);
    
    MiniMapMailFrame:ClearAllPoints();
    MiniMapMailFrame:SetPoint("BOTTOMRIGHT", MinimapCluster, "BOTTOMRIGHT", 3, 36);
    
    -- time manager
    
    TimeManagerClockButton:ClearAllPoints();
    --TimeManagerClockButton:SetPoint("TOP", MinimapCluster, "BOTTOM", -10, 47);
    TimeManagerClockButton:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", 0, -2.5);

    TimeManagerClockButton:SetParent(UIParent);

    TimeManagerFrame:ClearAllPoints();
    TimeManagerFrame:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -100, 150);
    CleanUI_SetBackdrop(TimeManagerClockButton, true, nil, 0, 5, 2, 0);

    CleanUI_ReplaceTextureWith("Interface\\TimeManager\\ClockBackground", TimeManagerClockButton, nil);
    
    TimeManagerClockButton:SetSize(68, 43);
    
    -- change right click
    TimeManagerClockButton:SetScript("OnClick", CleanUI_TimeManagerClockButton_OnClick);

    TimeManagerClockTicker:SetFontObject(NumberFontNormalLarge)
    
    -- recording button
    
    --MiniMapRecordingButton:ClearAllPoints();
    --MiniMapRecordingButton:SetPoint("LEFT", TimeManagerClockButton, "RIGHT", 0, 0);
    
    -- landing page 
    --GarrisonLandingPageMinimapButton:SetScale(1);
    --GarrisonLandingPageMinimapButton:ClearAllPoints();
    --GarrisonLandingPageMinimapButton:SetPoint("TOP", MinimapCluster, "BOTTOMRIGHT", -25, 45);
    
    -- queue status button
    QueueStatusMinimapButton:SetScale(1.2);
    QueueStatusMinimapButton:ClearAllPoints();
    QueueStatusMinimapButton:SetPoint("TOP", MinimapCluster, "BOTTOMLEFT", 20, 38);
    QueueStatusMinimapButtonBorder:Hide();
    
    -- instance difficulty
    MiniMapInstanceDifficulty:SetPoint("TOPLEFT", 24, 2);
    GuildInstanceDifficulty:SetPoint("TOPLEFT", 24, 2);
    MiniMapChallengeMode:SetPoint("TOPLEFT", 24, -3);
    
    -- change icons
    
    WorldMapFrame.BorderFrame.portrait:SetTexture("Interface\\worldmap\\UI-World-Icon");
    MiniMapMailIcon:SetTexture("Interface\\mailframe\\mail-icon");
end

function CleanUI_UpdateMiniMapCoords()
    local player_x, player_y = CleanUI_GetPlayerMapPosition();
    CUIWorldMiniMapFrameCoords:SetText(player_x.."|cffffffff/|r"..player_y);
end

function CleanUI_TimeManagerClockButton_OnClick(self, button)
    if (button == "LeftButton") then
        TimeManagerClockButton_OnClick(self, button);
        return;
    end
    
    CleanUI_ShowContextMenu(self);
end

function mmm()
    MinimapCluster:SetParent(UIParent);

    MinimapBackdrop:SetParent(MinimapCluster);
    Minimap:SetParent(MinimapCluster);
    GameTimeFrame:SetParent(MinimapCluster);
    
    MinimapCluster:SetAlpha(0.2);
end

-- collect MinimapButtons

local excludeButtons = { "MinimapZoneTextButton", "MiniMapWorldMapButton", "MiniMapTrackingDropDownButton", "MiniMapTrackingButton",
                         "MinimapZoomIn", "MinimapZoomOut", "GarrisonLandingPageMinimapButton", "QueueStatusMinimapButton",
                         "GameTimeFrame", "TimeManagerClockButton" };

local excludeFragments = { "GatherMate" };

local function exclude(name)
    if (name) then
        if (tContains(excludeButtons, name)) then
            return true;
        end

        for i, fragment in ipairs(excludeFragments) do
            if (string.find(name, fragment)) then
                return true;
            end
        end

        return false;
    end

    return true;
end

function CleanUI_CollectMinimapButtons()
    if (not CleanUISavedVariables.collectMinimapButtons) then
        return;
    end

    local buttons = CleanUI_GetChildrenWithType(MinimapCluster, "button");

    local offset = 0;
    local parent = nil;

    for i, button in ipairs(buttons) do
        if (not exclude(button:GetName()) and button:GetObjectType() == "Button" ) then
            button:SetScript("OnDragStart", nil);
            button:SetScript("OnDragStop", nil);

            button:SetScale(0.75);
            button:ClearAllPoints();

            button:SetParent(MinimapCluster);

            if (button:IsVisible()) then
                if (parent == nil) then
                    button:SetPoint("TOP", MinimapCluster, "TOPLEFT", 22, -35);
                else
                    button:SetPoint("TOP", parent, "BOTTOM", 0, 0);
                end

                parent = button;
            end
        end
    end
end

function CleanUI_InitLocationBar()
    -- create micro button bar
    local locationBar = CreateFrame("Frame", "CUI_LocationBar", UIParent, BackdropTemplateMixin and "BackdropTemplate");
    CleanUI_SetGradientBackdrop(locationBar);

    locationBar:SetSize(650, 54);
    locationBar:SetFrameStrata("BACKGROUND");
    
    locationBar:SetPoint("BOTTOM", UIParent, "TOP", 0, -5);
    
    locationBar:EnableMouse(true);
    locationBar:HookScript("OnEnter", CleanUI_LocationBar_OnEnter);
    locationBar:HookScript("OnLeave", CleanUI_LocationBar_OnLeave);
    locationBar:HookScript("OnMouseUp", ToggleWorldMap);

    CleanUI_Minimap_UpdateLocationBar();
end

function CleanUI_LocationBar_OnEnter()
    CUI_LocationBar:SetPoint("BOTTOM", UIParent, "TOP", 0, -52);
end

function CleanUI_LocationBar_OnLeave()
    CUI_LocationBar:SetPoint("BOTTOM", UIParent, "TOP", 0, -5);
end

function CleanUI_Minimap_UpdateLocationBar()
    if (not CUI_LocationBar) then
        return;
    end

    if (not CUI_LocationBar.zoneInfo) then
        CUI_LocationBar.zoneInfo = CUI_LocationBar:CreateFontString(null, "ARTWORK");
        CUI_LocationBar.zoneInfo:SetFontObject("GameFontNormalHuge");
        CUI_LocationBar.zoneInfo:SetSize(16, 16);
        CUI_LocationBar.zoneInfo:ClearAllPoints();
        CUI_LocationBar.zoneInfo:SetPoint("TOPLEFT", CUI_LocationBar, "TOPLEFT", 0, 0);
        CUI_LocationBar.zoneInfo:SetPoint("BOTTOMRIGHT", CUI_LocationBar, "BOTTOMRIGHT", 0, 15);
        CUI_LocationBar.zoneInfo:SetJustifyH("CENTER");
    end

    if (not CUI_LocationBar.additionalInfo) then
        CUI_LocationBar.additionalInfo = CUI_LocationBar:CreateFontString(null, "ARTWORK");
        CUI_LocationBar.additionalInfo:SetFontObject("GameFontNormalLarge");
        CUI_LocationBar.additionalInfo:SetSize(16, 16);
        CUI_LocationBar.additionalInfo:ClearAllPoints();
        CUI_LocationBar.additionalInfo:SetPoint("TOPLEFT", CUI_LocationBar.zoneInfo, "BOTTOMLEFT", 0, 0);
        CUI_LocationBar.additionalInfo:SetPoint("BOTTOMRIGHT", CUI_LocationBar, "BOTTOMRIGHT", 0, 12);
        CUI_LocationBar.additionalInfo:SetJustifyH("CENTER");
    end

	local pvpType, isSubZonePvP, factionName = GetZonePVPInfo();

	if ( pvpType == "sanctuary" ) then
		CUI_LocationBar.zoneInfo:SetTextColor(0.41, 0.8, 0.94);
	elseif ( pvpType == "arena" ) then
		CUI_LocationBar.zoneInfo:SetTextColor(1.0, 0.1, 0.1);
	elseif ( pvpType == "friendly" ) then
		CUI_LocationBar.zoneInfo:SetTextColor(0.1, 1.0, 0.1);
	elseif ( pvpType == "hostile" ) then
		CUI_LocationBar.zoneInfo:SetTextColor(1.0, 0.1, 0.1);
	elseif ( pvpType == "contested" ) then
		CUI_LocationBar.zoneInfo:SetTextColor(1.0, 0.7, 0.0);
	else
		CUI_LocationBar.zoneInfo:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
    end
    

    local zoneName = GetZoneText();
    local subzoneName = GetSubZoneText();

    local zoneInfo = zoneName;

    if (subzoneName and subzoneName ~= "") then
        zoneInfo = zoneName.." - "..subzoneName;
    end

    CUI_LocationBar.zoneInfo:SetText(zoneInfo);
    
    -- additional info

    local additionalInfo = nil;

    if ( pvpType == "friendly" ) then
        if (factionName and factionName ~= "") then
            additionalInfo = format(FACTION_CONTROLLED_TERRITORY, factionName);
            CUI_LocationBar.additionalInfo:SetTextColor(0.1, 1.0, 0.1);
        end
    elseif ( pvpType == "hostile" ) then
        if (factionName and factionName ~= "") then
            additionalInfo = format(FACTION_CONTROLLED_TERRITORY, factionName);
            CUI_LocationBar.additionalInfo:SetTextColor(1.0, 0.1, 0.1);
        end
    elseif ( pvpType == "contested" ) then
        additionalInfo = CONTESTED_TERRITORY;
        CUI_LocationBar.additionalInfo:SetTextColor(1.0, 0.7, 0.0);
    elseif ( pvpType == "combat" ) then
        additionalInfo = COMBAT_ZONE;
        CUI_LocationBar.additionalInfo:SetTextColor(1.0, 0.1, 0.1);
    end


    if (additionalInfo) then
        CUI_LocationBar.zoneInfo:SetPoint("BOTTOMRIGHT", CUI_LocationBar, "BOTTOMRIGHT", 0, 15);
        CUI_LocationBar.additionalInfo:SetText(additionalInfo);
    else
        CUI_LocationBar.zoneInfo:SetPoint("BOTTOMRIGHT", CUI_LocationBar, "BOTTOMRIGHT", 0, 0);
        CUI_LocationBar.additionalInfo:SetText("");
    end
end


