local function findCorner(angle)
    local x = 0.5 + math.cos(angle) / sqrt(2);
    local y = 0.5 + math.sin(angle) / sqrt(2);
    return x, y;
end

function CleanUI_InitRangePointer()
    CleanUI_RangePointerHookEvents();
    CleanUI_RegisterRangePointerEvents();
end

function CleanUI_RangePointerHookEvents()
    hooksecurefunc("PartyMemberFrame_OnUpdate", CleanUI_PartyMemberFrame_OnUpdate);
end

function CleanUI_RegisterRangePointerEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");

    frame:HookScript("OnEvent", CleanUI_RangePointerOnEvent);
end

function CleanUI_RangePointerOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
    end
end

function CleanUI_PartyMemberFrame_OnUpdate(self, elapsed)
    CleanUI_UpdateArrow(self);
end

function CleanUI_UpdateArrow(unitframe)
    CleanUI_CreateArrowFrame(unitframe);
    
    -- get unit
    local unit = unitframe.unit;

    local isAFK = UnitIsAFK(unit);
    local isDead = UnitIsDeadOrGhost(unit);
    local isOnline = UnitIsConnected(unit);
    local isInRange = UnitInRange(unit);
    local inInstance, instanceType = IsInInstance();

    if (inInstance) then
        unitframe.arrowFrame:Hide();
        return;
    end

    local alpha = 0.8;
    
    if (isInRange and not MouseIsOver(unitframe)) then
        alpha = 0.1;
    end

    if (not isOnline or isAFK or isDead) then
        alpha = 0.3;
    end

    -- get coordinates
    local playerX, playerY, playerZ, playerMapId = UnitPosition("player");
    local unitX, unitY, unitZ, unitMapId = UnitPosition(unit);

    -- not the same map
    if (playerMapId ~= unitMapId) then
        unitframe.arrowFrame:Hide();
        return;
    end

    -- no coordinates
    if (not playerX or not unitX) then
        unitframe.arrowFrame:Hide();
        return;
    end

    local oldAlpha = unitframe.arrowFrame:GetAlpha();
    if (alpha < oldAlpha) then
        UIFrameFadeOut(unitframe.arrowFrame, 0.2, oldAlpha, alpha);
    elseif (alpha > oldAlpha) then
        UIFrameFadeIn(unitframe.arrowFrame, 0.2, oldAlpha, alpha);
    end

    unitframe.arrowFrame:Show();

    local arrowTexture = unitframe.arrowFrame.texture;
        
    -- text and color
    local distance = sqrt(UnitDistanceSquared(unit));
    unitframe.arrowFrame.distance:SetText(format("%.0f", distance));
    
    if (distance < 20) then
        arrowTexture:SetVertexColor(0, 1, 0);
    elseif (UnitInRange(unit)) then
        arrowTexture:SetVertexColor(1, 1, 1);
    else
        arrowTexture:SetVertexColor(1, 0, 0);
    end

    -- rotate
    local dy = unitY - playerY;
    local dx = unitX - playerX;

    local facing = GetPlayerFacing();

    if (dx and dy and facing) then
        local angle = atan2(dy, dx) * PI / 180 - facing;

        local ulx, uly = findCorner(angle + 1.25 * PI);
        local llx, lly = findCorner(angle + 0.75 * PI);
        local urx, ury = findCorner(angle - 0.25 * PI);
        local lrx, lry = findCorner(angle + 0.25 * PI);
        
        arrowTexture:SetTexCoord(ulx, uly, llx, lly, urx, ury, lrx, lry);
    end
end

function CleanUI_CreateArrowFrame(unitframe)
    if (unitframe.arrowFrame) then
        return unitframe.arrowFrame;
    end

    unitframe.arrowFrame = CreateFrame("Frame", unitframe:GetName().."Arrow", unitframe);
    unitframe.arrowFrame:Hide();
    unitframe.arrowFrame.timer = 0;
    unitframe.arrowFrame:SetFrameStrata("LOW");    
    unitframe.arrowFrame:SetSize(18, 18);

    unitframe.arrowFrame:EnableMouse(false);
    
    unitframe.arrowFrame.texture = unitframe.arrowFrame:CreateTexture(nil, "OVERLAY");
    unitframe.arrowFrame.texture:SetAllPoints(true);
    unitframe.arrowFrame.texture:SetTexture("Interface\\MiniMap\\MiniMap-QuestArrow");
    
    unitframe.arrowFrame.distance = unitframe.arrowFrame:CreateFontString(nil, "OVERLAY", "CuiFont_ExtraSmall");
    unitframe.arrowFrame.distance:SetPoint("TOP", unitframe.arrowFrame, "BOTTOM", 0, 4);
    unitframe.arrowFrame.distance:SetScale(0.8)
    
    unitframe.arrowFrame.unit = unitframe.unit;
    
    unitframe.arrowFrame:ClearAllPoints();
    unitframe.arrowFrame:SetPoint("LEFT", unitframe, "RIGHT", -10, 5);

    return unitframe.arrowFrame;
end